﻿// Copyright (c) 2022 Jon P Smith, GitHub: JonPSmith, web: http://www.thereformedprogrammer.net/
// Licensed under MIT license. See License.txt in the project root for license information.

using System.Diagnostics;
using AuthPermissions.SupportCode.AddUsersServices;
using Example6.MvcWebApp.Sharding.Models;
using Example6.SingleLevelSharding.Services;
using Microsoft.AspNetCore.Mvc;
using StatusGeneric;

namespace Example6.MvcWebApp.Sharding.Controllers
{
    public class HomeController : Controller
    {
        private readonly ILogger<HomeController> _logger;

        public HomeController(ILogger<HomeController> logger)
        {
            _logger = logger;
        }

        public IActionResult Index(string message)
        {
            ViewBag.Message = message;

            if (AddTenantNameClaim.GetTenantNameFromUser(User) == null)
                return View(new AppSummary());

            return RedirectToAction("Index", "Invoice");
        }

        public ActionResult ErrorDisplay(string errorMessage)
        {
            return View((object)errorMessage);
        }

        public async Task<IActionResult> Privacy([FromServices] ISignInAndCreateTenant userRegisterInvite)
        {
            AddNewUserDto newUserData = new()
            {
                Email = "request.Email@gmail.com",
                UserName = "request.UserName",
                Password = "request.Password123!",
                IsPersistent = false,
            };

        AddNewTenantDto newTenantData = new()
        {
            TenantName = "request.TenantName",
            Version = "Pro"
        };

        StatusGenericHandler<string> status = new();

        IStatusGeneric<AddNewUserDto> userStatus = await userRegisterInvite.SignUpNewTenantWithVersionAsync(
            newUserData,
            newTenantData,
            CreateTenantVersions.TenantSetupData);

            return View();
        }

        [ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }
    }

    public static class CreateTenantVersions
{
    public static readonly MultiTenantVersionData TenantSetupData = new()
    {
        TenantRolesForEachVersion = new Dictionary<string, List<string>?>()
        {
            { "Free", new List<string> { "Tenant Admin" } },
            { "Pro", new List<string> { "Tenant Admin", "Pro" } },
            { "Enterprise", new List<string> { "Tenant Admin", "Pro", "Enterprise" } },
        },
        TenantAdminRoles = new Dictionary<string, List<string>>()
        {
            { "Free", new List<string> { "Tenant Admin" } },
            { "Pro", new List<string> { "Tenant Admin" } },
            { "Enterprise", new List<string> { "Tenant Admin" } }
        },
        HasOwnDbForEachVersion = new Dictionary<string, bool?>()
        {
            { "Free", false },
            { "Pro", false },
            { "Enterprise", true }
        }
    };
}
}
