﻿// Copyright (c) 2023 Jon P Smith, GitHub: JonPSmith, web: http://www.thereformedprogrammer.net/
// Licensed under MIT license. See License.txt in the project root for license information.

using AuthPermissions.BaseCode.DataLayer.EfCode;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using TestSupport.Attributes;
using TestSupport.Helpers;
using Xunit.Abstractions;
using Xunit.Extensions.AssertExtensions;

namespace Test.UnitTests.TestAuthPermissions;

public class TestVersion2MigrateDataKey
{
    private readonly ITestOutputHelper _output;

    public TestVersion2MigrateDataKey(ITestOutputHelper output)
    {
        _output = output;
    }

    [RunnableInDebugOnly]
    public void TestMigrateVersion1AuthPermissionsDbContext()
    {
        //SETUP
        //Needs special database only on Jon P Smith's PC
        var connectionString = AppSettings.GetConfiguration().GetConnectionString("Version1Example4");
        var builder = new DbContextOptionsBuilder<AuthPermissionsDbContext>();
        builder.UseSqlServer(connectionString);
        using var context = new AuthPermissionsDbContext(builder.Options);

        using var transaction = context.Database.BeginTransaction();
        foreach (var tenant in context.Tenants)
        {
            _output.WriteLine($"{tenant.ParentDataKey}, \t {tenant.TenantFullName}");
        }

        //ATTEMPT
        var sql = "authp.Tenants".CreateVersion2DataKeyUpdateSql("ParentDataKey");
        context.Database.ExecuteSqlRaw(sql);

        //VERIFY
        context.ChangeTracker.Clear();
        var updatedTenants = context.Tenants.ToList();
        _output.WriteLine("------------------------------------------");
        foreach (var tenant in updatedTenants)
        {
            _output.WriteLine($"{tenant.ParentDataKey}, \t {tenant.TenantFullName}");
        }
        updatedTenants.Where(x => x.ParentDataKey != null).All(x => x.ParentDataKey[0] != '.').ShouldBeTrue();
    }
}